// dw_dbstructs.h

// for DWEdit 1.1.22.1
//

#ifndef _DW_DBSTRUCTS_H_
#define _DW_DBSTRUCTS_H_
#pragma once

#include <assert.h>

#define BLOCKLENGTH 180
#define DATASIZE 116

#define MAX_DB_DISPLAYNAME_LEN	40
#define MAX_DB_NAME_LEN			20
#define MAX_DB_CLASSSNAME_LEN	18
#define MAX_DB_COUNTRYNAME_LEN	4
//#define MAX_DB_SENSORNAME_LEN   60

#define ENTITY_NUM_SENSORS		15	// available sensor slots in object database
#define ENTITY_NUM_LAUNCHERS	30	// available launcher slots in object
									// database
#define LAUNCHER_NUM_WEAPONS	4	// available weapons types per launcher
#define ENTITY_NUM_AIRSTRIPS	10	// available airstrips (0 = unavailable)
#define ENTITY_NUM_MISSIONS		7
#define ENTITY_NUM_SHIPBASEDAIRCRAFT	7// damn long word ;-(

#define ENTITY_NUM_THRUSTS	    5   //
#define NUMPROFILEFREQS         5
#define DB_MAX_DETECTIONCURVES	6

// DBOBjectFlags
#define MC_AAM					1		// This object is an air-to-air missile
#define	MC_ASM					2		// This object is an air-to-surface missile
#define	MC_SSM					4		// This object is a surface-to-surface missile
#define	MC_SAM					8		// This object is a surface-to-air missile
#define MC_TANKER				16		// This aircraft is a tanker capable of refueling other aircraft
#define	MC_TERRAIN_FOLLOW		32 		// This weapon can terrain follow
#define	MC_GUIDED				64		// This weapon is guided by the firing platform
#define	MC_THROWN				128		// This weapon has no propulsion, but is thrown by the firing platform
#define MC_REFUELABLE			256		// This aircraft can refuel inflight
#define MC_FEEDBACK				512		// This weapon feeds its target info back to the firing platform
#define MC_PRELOCK				1024	// This weapon must acquire before it fires
#define MC_SMOKELESS			2048	// This weapon doesn't leave a smoke trail
#define MC_REATTACK				4096	// This weapon can reattack if it misses
#define MC_REMOTE				8192	// This weapon can be fired on remote targeting info
#define MC_SNAPSHOT				16384	// This weapon receives a snapshot of the target at time of fire
#define MC_BALLISTIC			32768	// This weapon follows a ballistic trajectory

// Weapons Target Types
#define	TC_SUB					1
#define	TC_SURF					2
#define	TC_AIR					4
#define	TC_HELO					8
#define	TC_LAND					16
#define	TC_TORPEDO				32
#define	TC_MINE					64
#define	TC_MISSILE				128

// SimTypes
#define ST_SUB					0
#define ST_SHIP					1
#define ST_TORP					2
#define ST_CM					3
#define ST_FLOAT				4
#define ST_ROCK					5
#define ST_MINE					6
#define ST_DEPTH				7   //DepthCharge
#define ST_PLANE				8
#define ST_HELO					9
#define ST_MISSILE				10
#define ST_BIO					11
#define ST_BULLET				12  //Projectile
#define ST_SONOBUOY				13
#define ST_UNKNOWN				14

enum eSimType
{
    kSTSub = 0,
    kSTShip,
    kSTTorp,
    kSTCM,
    kSTFloat,
    kSTRock,
    kSTMine,
    kSTDepth,
    kSTPlane,
    kSTHelo,
    kSTMissile,
    kSTBio,
    kSTBullet,
    kSTSonobuoy,
    kSTMaxIndex,
    kSTForceDword = 0x7fffffff
};


const char* GetSimTypeString( eSimType eType );


#define MAX_ANIM_PARTS 15

enum eAnimType
{
    kATNone = 0,
    kATFrontFin = 2,
    kATTomahawkFin = 3,
    kATBioFin1 = 4,
    kATBioFin2 = 5,
    kATRotX = 88,
    kATRotY = 89,
    kATRotZ = 90,
    kATStandard = 99,
    kATForceDword = 0x7fffffff
};
const char* GetAnimTypeString( eAnimType eType );


// DBUser1Flags
#define U_VAL1					1
#define U_VAL2					2
#define U_VAL3					4
#define U_VAL4					8
#define U_VAL5					16
#define U_VAL6					32
#define U_VAL7					64
#define U_VAL8					128



// Objecttype Database Values [danmn unused entries will
// enforce conversion for comboboxData]
/*
#define OT_UNKNOWN				0 //#define OT_UNUSED1				0
#define OT_SUB					1
#define OT_SURF					2
#define OT_AIR					3
#define OT_HELO					4
#define OT_UNUSED2				5 //MOBILE_LAND
#define OT_STAT					6
#define OT_MINE					7
#define OT_WEAPON				8
#define OT_MISSILE				9
#define OT_SHELL				10
#define OT_BULLET				11
#define OT_DOWNEDPILOT			12
*/

enum eObjectType
{
    kOTUnkown = 0,
    kOTSub,
    kOTSurf,
    kOTAir,
    kOTHelo,
    kOTUnused2,//MOBILE_LAND
    kOTStat,//LAND
    kOTMine,
    kOTWeapon,
    kOTMissile,
    kOTShell,
    kOTBullet,
    kOTDownedPilot,
    kOTType13,
    kOTType14,
    kOTIceBerg,
    kOTMaxIndex,
    kOTForceDword = 0x7fffffff
};
const char* GetObjectTypeString( eObjectType eType );

enum eSSP {
    kSSPBottomLimited = 0,
    kSSPSurfaceDuct = 1,
    kSSPConvergenceZone = 2,
    kSSPMaxIndex,
    kSSPForceDword = 0x7fffffff
};
const char* GetSSPTypeString( eSSP eType );

enum eBottomType {
    kBTRock = 0,
    kBTMud = 1,
    kBTSand = 2,
    kBTMaxIndex,
    kBTForceDword = 0x7fffffff
};
const char* GetBottomTypeString( eBottomType eType );

enum eWeatherType {
    kWTClear     = 0,
    kWTCloads    = 1,
    kWTOvercast  = 2,
    kWTRain      = 3,
    kWTMaxIndex,
    kWTForceDword = 0x7fffffff
};
const char* GetWeatherTypeString( eWeatherType eType );

// Objecttype conversion(for Combobox) [Wrapper for hardcoded ComboBoxData]
#define OT_CONV_SUB				0
#define OT_CONV_SURF			1
#define OT_CONV_AIR				2
#define OT_CONV_HELO			3
#define OT_CONV_STAT			4
#define OT_CONV_MINE			5
#define OT_CONV_WEAPON			6
#define OT_CONV_MISSILE			7
#define OT_CONV_SHELL			8
#define OT_CONV_BULLET			9
#define OT_CONV_DOWNEDPILOT		10
//#define OT_CONV_TYPE13  		13
//#define OT_CONV_TYPE14  		14
#define OT_CONV_ICEBERG  		11
int ObjectType2Ctrl( eObjectType eType );
eObjectType Ctrl2ObjectType( int ObjectTypeCTRL );

// Countermeasures carried
#define CM_CHAFF				1
#define CM_IR_FLARES			2
#define CM_ACOUSTICS			4
#define CM_TOWED_ACOUSTICS		8   // Seems to be the ONLY Countermeasure used by NavalSimEngine
#define CM_ECM_JAMMER			16
#define CM_CIWS					32

enum eSensorClass
{
    kSCNone = 0,
    kSCActive,
    kSCActiveIntercept,
    kSCESM,
    kSCVisual,
    kSCMAD,
    kSCIR,
    kSCPassive,
    kSCRadar,
    kSCFCRadar,
    kSCIFF,
    kSCCM,
    kSCJammer,
    /*
    kSCCM,          // IFF
    kSCJammer,      // CM
    kSCSonoBuoy,
    kSCIFF,
    */
    kSCMaxIndex,
    kSCForceDword = 0x7fffffff
};
const char* GetSensorClassString( eSensorClass eType );

/*
// Sensor Classes
#define SC_NONE					0
#define SC_ACTIVE				1
#define SC_ACTIVE_INTERCEPT		2
#define SC_ESM					3
#define SC_VISUAL				4
#define SC_MAD					5
#define SC_IR					6
#define SC_PASSIVE				7
#define SC_RADAR				8
#define SC_FC_RADAR				9
#define SC_CM					10
#define SC_JAMMER				11
#define SC_SONOBUOY				12
#define SC_IFF					13
*/

// Sensor Affectors
#define DA_AFFECTEDBYRANGE	        1       //1
#define DA_THERMALLAYER    	        2       //2
#define DA_SEASTATE        	        4       //3
#define DA_AFFECTEDBYASPECT	        8       //4 (former UNKNOWN8)
#define DA_TARGETMASS      	        16      //5
#define DA_TARGETNOISE     	        32      //6
#define DA_PLATFORMNOISE   	        64      //7
#define DA_AFFECTEDBYGROUNDNOISE    128     //8 Ground Noise (former UNKNOWN128 IgnoreLandTarget)
#define DA_EARTHCURVATURE  	        256     //9
#define DA_AMBIENTLIGHT    	        512     //10
#define DA_UNKNOWN1024     	        1024    //11 DetachFromParent
#define DA_MIRROR	     	        2048    //12
#define DA_DOUBLESIDEDCONE 	        4096    //13 DoubleSensorCone
#define DA_RPT_TGTBEARING  	        8192    //14 (former Jammable)
#define DA_RPT_TGTRANGE    	        16384   //15 (former Activly Updated UNKNOWN16384 )
#define DA_RPT_TGTCRS               32768   //16 Reports TgtCrs (former UNKNOWN32768)
#define DA_RPT_TGTSPD               65536   //17 Reports TgtSpeed
#define DA_RPT_TGTALT               131072  //18
#define DA_RPT_TGTFREQ              262144  //19
#define DA_RPT_TGT003               524288  //20


#define DA_DATA16_1			1    
#define DA_DATA16_2         2    
#define DA_DATA16_4         4    
#define DA_DATA16_8         8    
#define DA_DATA16_16        16   
#define DA_DATA16_32        32   
#define DA_DATA16_64        64   
#define DA_DATA16_128       128
#define DA_DATA16_256       256  
#define DA_DATA16_512       512  
#define DA_DATA16_1024      1024 
#define DA_DATA16_2048      2048 
#define DA_DATA16_4096      4096 
#define DA_DATA16_8192      8192 
#define DA_DATA16_16384     16384
#define DA_DATA16_32768     32768


// enable tight packing to avoid wrong auto align
#pragma pack( push, before_db_include )
#pragma pack(1)

///////////////////////////////////////////////////////////////////////////////
// Entity DB
///////////////////////////////////////////////////////////////////////////////
struct scEntityInfo
{
	short	Index;//0x00
	short	ShipClassIndex;//0x02
	char	name[MAX_DB_NAME_LEN];//0x04
	short	data01;//0x18 ArrayIndexOfAssignedObject (Integer)
	short	data02;//0x1a
	short	data03;//0x1c
	short	data04;
	short	data05;
	short	shipNumber;
	char	typeName[MAX_DB_NAME_LEN];
	short	data06;//possibly unused
}; // size 0x3A

struct dbEntityInfo
{
    dbEntityInfo();

	short	sID;                        // 0x00 EntityID
	short	sShipClassID;               // 0x02 ShipClassID
	short	sShipNumber;                // 0x04 // this may be an int !???
    short	sData06;                    // 0x06
	char	szName[MAX_DB_NAME_LEN];    // 0x08
	short	sObjectID;                  // 0x1C
	short	sData1E;                    // 0x1E
	short	sData20;                    // 0x20
	short	sData22;	                // 0x22
    short	sData24;	                // 0x24
	char	szTypeName[MAX_DB_NAME_LEN];// 0x26
}; // size 0x3A
void ConvertSC2DW( dbEntityInfo* pdbEntity, scEntityInfo* pscEntity);





///////////////////////////////////////////////////////////////////////////////
// Launcher DB
///////////////////////////////////////////////////////////////////////////////
struct scLauncherInfo
{
	short	Index;							// unused
	short	tubes;							// number of tubes, rails, barrels,
											// etc in launcher
	short	center;							// orientation of centered launcher
											// relative to platform
	short	cone;							// 1/2 range of firing cone in
											// degrees (e.g. 360 degree
											// coverage = 180,
											// forward coverage = 90)
	short	pRel;							// pitch relative to platform
	short	hRel;							// heading relative to platform
	short	fixed;							// bit flags, 1=Rotatable,
											// 4=Requires Fire Control Radar
	short	load;							// time in seconds to load launcher
	short	swap;							// time in seconds to swap in
											// different weapon, if applicable
	short	maxGuidedWeaponCap;				// max guided weapons controllable
											// from this launcher
	short	rdsPerMin;						// rounds per minute firing
											// capability)
	short	AdditionalClips;				// Additional ammon clips? or
											// Alignment ?
	float	centx,centy,centz;				// launcher center x,y,z relative
											// to platform center
	float	x1,y1,z1;						// upper left corner of launcher
											// face
	float	x2,y2,z2;						// lower right corner of launcher
											// face
	short	rows;							// rows of tubes
	short	cols;							// columns of tubes
	short	quant[LAUNCHER_NUM_WEAPONS];	// 4 quantity of weapons, matching
											// weap array above
	short	weapPtr[LAUNCHER_NUM_WEAPONS];	// 4references to Index in Entitydb
											// database
	char	soundFile[MAX_DB_NAME_LEN];		// firing sound
	char	name[MAX_DB_NAME_LEN];			// 20 UNIQUE name
};// size 0x78


struct dbLauncherInfo
{    
	short	sID;							// 0x00 unused
	short	sNumTubes;							// 0x02 number of tubes, rails, barrels,
											//      etc in launcher
	short	sCenter;						// 0x04 orientation of centered launcher
											//      relative to platform
	short	sCone;							// 0x06 1/2 range of firing cone in
											//      degrees (e.g. 360 degree
											//      coverage = 180,
											//      forward coverage = 90)
	short	sPitchRel;						// 0x08 pitch relative to platform
	short	sHeadingRel;     				// 0x0A heading relative to platform
	short	sFixed;							// 0x0C bit flags, 1=UnRotatable,
											//      4=Requires Fire Control Radar
	short	sLoadTime;						// 0x0E time in seconds to load launcher
	short	sSwapTime;						// 0x10 time in seconds to swap in
											//      different weapon, if applicable
    
    short   sData12;                        // 0x12
	short	sRdsPerMin;						// 0x14 rounds per minute firing
											//      capability)
	short	sAdditionalClips;				// 0x16 Additional ammon clips? or											
    short	sNumRows;						// 0x18 rows of tubes
	short	sNumCols;						// 0x1A columns of tubes
    short	sWpnQuant[LAUNCHER_NUM_WEAPONS];// 0x1C 4 quantity of weapons, matching
											// weap array above ( 1C 1E 20 22 )
	short	sWpnID[LAUNCHER_NUM_WEAPONS];	// 0x24 4 references to Index in Entitydb
											// database ( 24 26 28 2A )
	float	fCenterX;              			// 0x2C launcher center x,y,z relative
    float   fCenterY;                       // 0x30 platform center
    float   fCenterZ;                       // 0x34 											
	float	fUpperLeftX;					// 0x38 upper left corner of launcher
    float   fUpperLeftY;                    // 0x3C face
    float   fUpperLeftZ;                    // 0x40
    float	fLowerRightX;					// 0x44 upper left corner of launcher
    float   fLowerRightY;                   // 0x48 face
    float   fLowerRightZ;                   // 0x4C
    char	szAnimPartName1[MAX_DB_NAME_LEN];// 0x50 part name in j3d model to animate
    char	szAnimPartName2[MAX_DB_NAME_LEN];// 0x64 part name in j3d model to animate    
    short	sMaxGuidedWeaponCap;			// 0x78 max guided weapons controllable
											//      from this launcher
    short   sData7A;                        // 0x7A
	char	szSoundFile[MAX_DB_NAME_LEN];	// 0x7C firing sound
	char	szName[MAX_DB_NAME_LEN];		// 0x90 20 chars UNIQUE name
};// size 0xA4
void ConvertSC2DW( dbLauncherInfo* pdbLauncher, scLauncherInfo* pscLauncher);


///////////////////////////////////////////////////////////////////////////////
// Object DB
///////////////////////////////////////////////////////////////////////////////
struct scObjectInfo
{
	unsigned short m_sID;//0x00 ObjectIndex				// Index ?
	char			name[MAX_DB_NAME_LEN];//0x02	// UNIQUE object name
	short			id;//0x16								// unused
	eObjectType		m_eObjectType;//0x18
	eSimType		m_eSimType;//0x1c			// sub,ship,torp,cm,float,rock,mine,depth
									// ,plane,helo,missile,bio,bullet
	char	AmbientSound[MAX_DB_NAME_LEN];//0x20// ambient sound file
	char	BBsoundFile[MAX_DB_NAME_LEN];//0x34// BroadBand sound file
	float	MastHeightx10;//0x48
	float	Lengthx10;//0x4c
	float	Widthx10;//0x50
	float	Draftx10;//0x54
	float	cd;//0x58								// drag coefficient
	int Mass;//0x5c
	//unsigned short	Mass;					// maybe the next WORD is a modifier
											// for the unit ????[tons default]
	//unsigned short	MassUnit;				// not sure how this works
	int		wpnMaxRange;//0x60
	int		wpnMinRange;//0x64//(Miss.=10000)
	int		fuel;//0x68// fuel carried, lbs
	short	buoyCount;//0x6c
	short	data16;//0x6e//(seems unused)
	short	armor;//0x70						// for non-weapons, armor value;
												// for weapons, warhead value
	char	defDoctrine[MAX_DB_NAME_LEN];//0x72 // filename of doctrine file

	char	mpASW;//0x86						// originally Array of 9 chars
	char	mpSUW;//0x87						
	char	mpAAW;//0x88
	char	mpStrike;//0x89
	char	mpRecon;//0x8a
	char	mpComms;//0x8b
	char	mpMineSweepBomber;//0x8c
	char	mpUnused1;//0x8d
	char	wpnEffectiveness;//0x8e
	char	wpnUnused1;//0x8f//padding for alignment

	int 	PointValue;//0x90
	short	maxSpeed;//0x94
	short	minSpeed;//0x96
	int		ClimbAngle;//0x98
	int		maxAlt;//0x9c
	int		minAlt;//0xa0
	float	turnRadius;//0xa4
	float	UnusedSL;//0xa8//(seems NOT unused)
	float	ActiveSonarSL;//0xac		// ActiveSonarEchoReturnFactor
	float	ActiveInterceptSL;//0xb0
	float	ESMSL;//0xb4
	float	VisualSL;//0xb8//31:Visual?
	float	MADSL;//0xbc// MAD Signal-Return-Strength
	float	InfraRedSL;//0xc0//IR Emission
	float	PassivSonarSL;//0xc4//SoundEmission
	float	RadarSL;//0xc8//RadarEcho?
	float	FCRadarSL;//0xcc
	float	sIFFSL;//0xd0
	float	sCMSL;//0xd4
	float	sJammerSL;//0xd8(seems unused)
	short	LauncherId[ENTITY_NUM_LAUNCHERS];//0xdc	// 30 references to index in
												// launcher database
	short	AirstripId[ENTITY_NUM_AIRSTRIPS];//0x118// 10 references to index in
												// airstrip database
	short	SensorId[ENTITY_NUM_SENSORS];//0x12c// min.15 references to index in
												// sensor database
	short   ProfileId;//0x14a // pointer Narrowband signature in Profile.qdp

	short   s3DObjectIndex;//0x14c // always 0xffff
	unsigned short  targetCaps;//0x14e	// bit flags, missiles only:
								// attributes, 1=AAM 2=ASM 4=SSM 8=SAM
								// 32=Terrain Follow 64=Guided 128=Thrown
	unsigned short  missileCaps;//0x150// bit flags, weapons only, object class that
								// can be targeted:
								// 1=Sub 2=Surf 4=Air 8=Helo 16=Land 128=Missile
	unsigned char	cmCaps;//0x152	// bit flags, countermeasures carried:
								// 1=Chaff 2=IRFlare 4=AcousticCM
								// 8=TowedAcousticCM 16=ECMJammer 32=CIWS
	unsigned char	DBUser1Flags;//0x153 // SCEdit uses this as CheckMarks
	unsigned char	DBUser2Flags; //0x154 SCEdit uses this as Byte Value
	unsigned char	data60;//0x155 // padding ?
	short	data61;//0x156 // always 0x00
}; // size 0x158

struct dbObjectInfo
{
    short   sID;                                // 0x000
    short   sMaxSpeed;                          // 0x002
    short   sMinSpeed;                          // 0x004
    short   sClimbAngle;                        // 0x006
    short   sArmorDamage;                       // 0x008
    short   sData0A;                            // 0x00A changing
    short   sUnusedSL;                          // 0x00C
    short   sActiveSonarSL;                     // 0x00E
    short   sActiveInterceptSL;                 // 0x010
    short   sESMSL;                             // 0x012
    short   sVisualSL;                          // 0x014
    short   sMADSL;                             // 0x016
    short   sInfraRedSL;                        // 0x018
    short   sPassivSonarSL;                     // 0x01A
    short   sRadarSL;                           // 0x01C
    short   sFCRadarSL;                         // 0x01E
    short   sIFFSL;                             // 0x020 always 0
    short   sCMSL;                              // 0x022 always 0
    short   sJammerSL;                          // 0x024 always 0       
    short	sLauncherID[ENTITY_NUM_LAUNCHERS];  // 0x028 ->30 references to index in
												//          launcher database
    short	sAirstripID[ENTITY_NUM_AIRSTRIPS];  // 0x062 ->10 references to index in
												//          airstrip database
    short	sSensorID[ENTITY_NUM_SENSORS];      // 0x076 ->15 references to index in
												//          sensor database
	short   sProfileID;                         // 0x094 pointer Narrowband signature in Profile.qdp
    short   s3DObjectIndex;                     // 0x096
    short   sThrustID[ENTITY_NUM_THRUSTS];      // 0x098 
    /*
    short   sDataF8;                            // 0x098 changing
    short   sDataFA;                            // 0x09A changing
    short   sDataFC;                            // 0x09C changing
    short   sDataFE;                            // 0x09E P3-Orion == 12, others 0
    short   sData100;                           // 0x0A0 always 0
    */
    unsigned short usTargetCaps;                // 0x0A2
    unsigned short usMissileCaps;               // 0x0A4
    short   sData106;                           // 0x0A6 always 0

    float   fMastHeightx10;                     // 0x0A8
    float   fLengthx10;                         // 0x0AC
    float   fWidthx10;                          // 0x0B0
    float   fDraftx10;                          // 0x0B4
    float   fCD;                                // 0x0B8
    float   fMass;                              // 0x0BC
    float   fTurnRadius;                        // 0x0C0

    eObjectType ObjectType;                     // 0x0C4 
    eSimType    SimType;                        // 0x0C8 
    
    int     nWpnMaxRange;                       // 0x0CC    
    int     nWireLength;                        // 0x0D0 new special Torpedo orientated running length
    int     nWpnMinRange;                       // 0x0D4
    int     nFuel;                              // 0x0D8
    int     nPointValue;                        // 0x0DC   	
    int     nMaxAltitude;                       // 0x0E0
    int     nMinAltitude;                       // 0x0E4
    
	char	szName[MAX_DB_NAME_LEN];            // 0x0E8 UNIQUE object name	
	char	szAmbientSound[MAX_DB_NAME_LEN];    // 0x0FC ambient sound file
	char	szBBsoundFile[MAX_DB_NAME_LEN];     // 0x110 BroadBand sound file
	char	szDoctrine[MAX_DB_NAME_LEN];        // 0x124 filename of doctrine file	
    char    cMisASW;                            // 0x138
    char    cMisSUW;                            // 0x139
    char    cMisAAW;                            // 0x13A
    char    cMisStrike;                         // 0x13B
    char    cMisRecon;                          // 0x13C
    char    cMisComms;                          // 0x13D
    char    cMisMineSweepBomber;                // 0x13E
    char    cWpnEffectiveness;                  // 0x13F
    char    cBuoyCount;                         // 0x140 DBBuoyFlags
    unsigned char cCMCaps;                      // 0x141
    char    cDBUser1Flags;                      // 0x142
    char    cDBUser2Flags;                      // 0x143
}; // size 0x144
void ConvertSC2DW( dbObjectInfo* pdbObject, scObjectInfo* pscObject);

///////////////////////////////////////////////////////////////////////////////
// Sensor DB
///////////////////////////////////////////////////////////////////////////////
struct  scSensorInfo
{
    short			m_sID;//0x00
    char			name[MAX_DB_DISPLAYNAME_LEN];//0x02
    short			RefMastID;//0x2A// Reference to Id in Mast DB
    unsigned int	SensorClass;//0x2C
    unsigned int	CounterDetectSensor;//0x30
    unsigned int	UpdateInterval;		//0x34
    unsigned int	DetectionCurveRange[DB_MAX_DETECTIONCURVES];//0x38
    //unsigned int	DetectionCurveRange2;//0x3c
    //unsigned int	DetectionCurveRange3;//0x40
    //unsigned int	DetectionCurveRange4;//0x44
    //unsigned int	DetectionCurveRange5;//0x48
    //unsigned int	DetectionCurveRange6;//0x4c
    unsigned short	DetectionCurveLevel[DB_MAX_DETECTIONCURVES];//0x50
    //unsigned short	DetectionCurveLevel2;//0x52
    //unsigned short	DetectionCurveLevel3;//0x54
    //unsigned short	DetectionCurveLevel4;//0x56
    //unsigned short	DetectionCurveLevel5;//0x58
    //unsigned short	DetectionCurveLevel6;//0x5A
    unsigned int	MaxOperationSpeed;//0x5c	// MaxOperationSpeed?
    int				MaxOperationAlt;//0x60
    int				MinOperationAlt;//0x64
    unsigned int	SonarFreqMin;//0x68//SonarFreq. Min
    unsigned int	SonarFreqMax;//0x6c//SonarFreq. Max
    int				SNR;//0x70// Signal to Noise Ratio negative vals 0 to -30, -30 is best...
    unsigned short	SensorAttrib;//0x74// Sensor Attributes defines look above
    unsigned short	data16a;//0x76//possibly unused
    short			sNumTrackers; // 0x78 (data17) Number of available Trackers
    
    char            cRootTracker;       // 0x7a (RngRingClr)
    char            cSierraPrefix;      // 0x7b

    unsigned int	ConeCenter;//0x7c
    unsigned int	SensorCone;//0x80
    //unsigned int	data19;
    float			PosX;//0x84
    //unsigned short	data19a;
    float			PosY;//0x88
    //unsigned int	data20;
    //unsigned short	data20a;
    float			PosZ;//0x8c
    //unsigned short	data21;
    //unsigned short	data21a;
    unsigned short	data22;//0x90
    unsigned short	data22a;//0x92
    float	data23;//0x94 fMinBeamWidth
    float	data24;//0x98 fMaxBeamWidth
    float	data25;//0x9c
    char			sName[MAX_DB_NAME_LEN];//0xa0
}; // size 0xb4

struct dbSensorInfo
{
    short			sID;                // 0x00
    short			sRefMastID;         // 0x02// Reference to Id in Mast DB
    unsigned short	usUpdateInterval;   // 0x04
    unsigned short	usDetectionCurveLevel[DB_MAX_DETECTIONCURVES];// 0x06 ^=6x ( 06 08 0A 0C 0e 10)    
    short           sMaxOperationSpeed; // 0x12
    short           sNumTrackers;       // 0x14 ndert sich!
    short           sConeRotAxisZ;      // 0x16 former sConeCenter
    short           sConeRotAxisY;      // 0x18 former sData18
    short           sConeRotAxisX;      // 0x1A former sData1A
    short           sConeAngleXZ;       // 0x1C former sData1C
    short       	sConeAngleXY;       // 0x1E former sSensorCone
    int             nRptAccuracy;       // 0x20 ndert sich!
    eSensorClass    SensorClass;        // 0x24
    eSensorClass    CounterDetectSensor;// 0x28
    unsigned int	nDetectionCurveRange[DB_MAX_DETECTIONCURVES];// 0x2C ^=( 2C 30 34 38 3C 40 )
    int             nMaxDetectionAlt;   // 0x44 ndert sich!
    int             nMinDetectionAlt;   // 0x48 ndert sich!
    int				nMaxOperationAlt;   // 0x4C
    int				nMinOperationAlt;   // 0x50
    unsigned int	nSonarFreqMin;      // 0x54 SonarFreq. Min
    unsigned int	nSonarFreqMax;      // 0x58 SonarFreq. Max
    int				nSNR;               // 0x5C Signal to Noise Ratio negative vals 0 to -30, -30 is best...
    float           fData60;            // 0x60 konst. 0!
    float			fPosX;              // 0x64
    float			fPosY;              // 0x68
    float			fPosZ;              // 0x6C    
    float           fMinBeamWidth;      // 0x70
    float           fMaxBeamWidth;      // 0x74
    char            cRootTracker;       // 0x78
    char            cSierraPrefix;      // 0x79
    //short         sColor;             // 0x78
    
    char			szName[MAX_DB_NAME_LEN];// 0x7A    
    char			szAnimPartName[MAX_DB_DISPLAYNAME_LEN];// 0x7A    
    char			szDisplayName[MAX_DB_DISPLAYNAME_LEN];//0xB6        
    short           sDataDE; // 0xDE konst. 0!->2A (42)
    unsigned int    uSensorAttribs;// 0xE0
}; // size 0xe4
void ConvertSC2DW( dbSensorInfo* pdbSensor, scSensorInfo* pscSensor);

///////////////////////////////////////////////////////////////////////////////
// 3DObject DB
///////////////////////////////////////////////////////////////////////////////
enum scAnimType
{
	ANIM_CONTINUOUS  = 0,					// part rotates around vector
											// continuously at rate given
	ANIM_PARAM		 = 1,         			// part's rotation is determined
											// by entity in sim
    ANIM_PARAM2		 = 2,					// ???
    ANIM_TOMAHAWKFIN = 3,					// ???
    ANIM_BIOFIN1     = 4,
    ANIM_BIOFIN2     = 5
};

struct scAnimatedPart
{
	float		vx; 					    // 0x00 rotational vector
    float		vy;     					// 0x04 rotational vector
    float		vz;     					// 0x08 rotational vector
	scAnimType	animType;					// 0x0C type of animation this part is
	float		da;							// 0x10 change in angle over time
	char		partName[MAX_DB_NAME_LEN];	// 0x14 part name in j3d model to animate
};// 0x28
// forward
struct AnimatedPart;
void ConvertSC2DW( AnimatedPart* pAnimPart, scAnimatedPart* pscAnimPart );

struct sc3DObjectInfo
{
	char	name[MAX_DB_NAME_LEN];			// 0x00 key for database, matches
											//      db_ObjectInfo name
	char	modelNameLow[MAX_DB_NAME_LEN];	// 0x14 name of low detail j3d model
	char	modelNameMed[MAX_DB_NAME_LEN];	// 0x28 name of medium detail j3d model
	char	modelNameHigh[MAX_DB_NAME_LEN];	// 0x3C name of high detail j3d model
	char	brokenName[3][MAX_DB_NAME_LEN];	// 0x50 names of broken parts
	float	waterLineRadius;				// 0x8C radius of hull at waterline
	short	freeVar01;                      // 0x90
	short	freeVar02;                      // 0x92
	short	freeVar03;                      // 0x94
	short	freeVar04;                      // 0x96
	int		numAnimParts;					// 0x98 number of animated parts
	scAnimatedPart anim[20];	            // 0x9C list of animated parts (20 * 0x28 = 0x3BC )
};//3dobject.g3d // size 0x03bc


struct AnimatedPart
{
	eAnimType   nData00;                    // 0x00 rotational vector ??
	eAnimType   nData04;                    // 0x04 animation track ( needs nData00 = 0 and fAnimSpeed = 0.0f )
	float		fAnimSpeed;					// 0x08 change in angle over time
	char		szPartName[MAX_DB_NAME_LEN];	// 0x0C part name in j3d model to animate
};//0x20

// db3DObjectInfo
struct db3DObjectInfo
{
	char	    szName[MAX_DB_NAME_LEN];		    // 0x00 key for database, matches
											        //      db_ObjectInfo name
	char	    szModelNameLow[MAX_DB_NAME_LEN];	// 0x14 name of low detail j3d model
	char	    szModelNameMed[MAX_DB_NAME_LEN];	// 0x28 name of medium detail j3d model
	char	    szModelNameHigh[MAX_DB_NAME_LEN];	// 0x3C name of high detail j3d model
	char	    szBrokenName[2][MAX_DB_NAME_LEN];	// 0x50//0x64-0x78 names of broken parts
	float	    fWaterLineRadius;				    // 0x78 radius of hull at waterline	
	int		    nNumAnimParts;					    // 0x7C number of animated parts
	AnimatedPart AnimParts[MAX_ANIM_PARTS];		    // 0x80 + 0x20* 0x0f = 0x1E0 list of animated parts
};// size 0x0260
void ConvertSC2DW( db3DObjectInfo* pdb3DObjectInfo, sc3DObjectInfo* psc3DObjectInfo );


///////////////////////////////////////////////////////////////////////////////
// Profile DB
///////////////////////////////////////////////////////////////////////////////
// used in game (SubCommand)
struct dbProfilToEntityRef
{
	unsigned char m_btIndex;
	short		  m_sEntityID;
};

struct scProfileInfo
{
	short	Index;
	char	name[MAX_DB_NAME_LEN];
	short	sFreqData[NUMPROFILEFREQS];
	BYTE	TurnsPerKnot;
	BYTE	Blades;
	short	data07;// unused
	short	data08;// unused
	short	data09;// unused
	short	data10;// unused
	short	data11;// unused
};// size 0x2C

struct dbProfileInfo
{
	short	sID;//00
    short	sFreqData[NUMPROFILEFREQS]; // 0x02-0C
    dbProfileInfo* pPrev;               // 0x00C
    dbProfileInfo* pNext;               // 0x10
    /*
    short	sData0C;                    // 0x0C unused possible dbProfileInfo* pPrev
    short	sData0E;                    // 0x0E unused        
    short	sData10;                    // 0x10 unused possible dbProfileInfo* pNext
    short	sData12;                    // 0x12 unused
    */

	char	szName[MAX_DB_NAME_LEN];    // 0x14	
	BYTE	cTurnsPerKnot;              // 0x28
	BYTE	cBlades;                    // 0x29
	short	sData2A;                    // 0x2A unused    
}; // size 0x2C
void ConvertSC2DW( dbProfileInfo* pdbProfile, scProfileInfo* pscProfile );

///////////////////////////////////////////////////////////////////////////////
// ShipClass DB
///////////////////////////////////////////////////////////////////////////////
struct scShipClassInfo 
{
	short	Index;
	short	CountryNameIndex;
	short	data02;
	short	data03;
	short	data04;
	char	name[MAX_DB_DISPLAYNAME_LEN];
	char	sName[MAX_DB_CLASSSNAME_LEN];
};// size 0x44

struct dbShipClassInfo
{
	short	sID;                // 0x00
	short	sCountryNameID;     // 0x02	
	short   sData04;            // 0x04
	short   sData06;            // 0x06
	short	sData08;            // 0x08
	char	szName[MAX_DB_DISPLAYNAME_LEN];
	char	szClassName[MAX_DB_CLASSSNAME_LEN];
};// size 0x44
void ConvertSC2DW( dbShipClassInfo* pdbShipClass, scShipClassInfo* pscShipClass );

///////////////////////////////////////////////////////////////////////////////
// CountryName DB
///////////////////////////////////////////////////////////////////////////////
struct scCountryNameInfo
{
	short	Index;
	char	name[MAX_DB_DISPLAYNAME_LEN];
	short	data01;
	short	data02;
	short	data03;
	short	data04;
	short	data05;
	char	sName[MAX_DB_COUNTRYNAME_LEN];
};// size 0x38

struct dbCountryNameInfo
{
	short	sID;
	char	szName[MAX_DB_DISPLAYNAME_LEN];// 0x02
	short	sData2A;            // 0x2A
	short	sData2C;            // 0x2C
    short	sData2E;            // 0x2E
	short	sData30;            // 0x30
	short	sData32;            // 0x32	
	char	szShortCut[MAX_DB_COUNTRYNAME_LEN]; // 0x34
};// size 0x38
void ConvertSC2DW( dbCountryNameInfo* pdbCountryName, scCountryNameInfo* pscCountryName );

///////////////////////////////////////////////////////////////////////////////
// Mast DB
///////////////////////////////////////////////////////////////////////////////
struct scMastInfo
{
	short	m_sID;//0x00
	char	name[MAX_DB_NAME_LEN];//0x02
	short	sDeployTime;//Deploytime
	short	sMaxSafeSpeed;//MaxSpeed
	short	sThickness;//Thickness(Wire)
	short	bouyoncy;
	short	data05;
	short	data06;
	short	data07;
	short	data08;
	short	data09;
	short	data10;
	short	data11;
	short	data12;
	short	data13;
	short	data14;
	char	soundFile[MAX_DB_NAME_LEN];		// ambient sound file
	short	data15;// padding Alignment
	int		TextID;						    // Text ID in Interface
											// TextDLL
	int		data17;// maps -> nData14
	float	centx;
	float	centy;
	float	centz;
	short   length;
	short	data22;// maps -> sData0C
}; // size 0x60

struct dbMastInfo
{
    dbMastInfo();

	short	sID;                                // 0x00
    short   sDeployTime;                        // 0x02
    short   sMaxSafeSpeed;                      // 0x04
    short   sThickness;                         // 0x06
    short   sBouyoncy;                          // 0x08
    short   sLength;                            // 0x0A
    short   sData0C;                            // 0x0C
    short   sData0E;                            // 0x0E
    int		nTextID;						    // 0x10 Text ID in Interface's TextDLL
    int     nData14;						    // 0x14		    
    float   fPosX;                              // 0x18
    float   fPosY;                              // 0x1C
    float   fPosZ;                              // 0x20
	char	szName[MAX_DB_NAME_LEN];            // 0x24
    char	sz3DFile[MAX_DB_DISPLAYNAME_LEN];   // 0x38    
    char	szSoundFile[MAX_DB_NAME_LEN];	    // 0x60 ambient sound file
    int     nData74;                            // 0x74 only for RadarMasts
}; // size 0x78
void ConvertSC2DW( dbMastInfo* pdbMastInfo, scMastInfo* pscMastInfo );

///////////////////////////////////////////////////////////////////////////////
// Mast DB
///////////////////////////////////////////////////////////////////////////////
#define OP_LAND			1
#define OP_TAKEOFF		2
#define OP_HELOPAD		4
#define OP_CATAPULT		8
#define OP_BIT5			16
#define OP_BIT6			32
#define OP_BIT7			64
#define OP_BIT8			128

struct scAirstripInfo
{
	short	m_sID;
	short	operation; // bitfield
	float	HangarX;
	float	HangarY;
	float	HangarZ;
	float	RunwayStartX;
	float	RunwayStartY;
	float	RunwayStartZ;
	float	RunwayEndX;
	float	RunwayEndY;
	float	RunwayEndZ;
	float	TaxiStartX;
	float	TaxiStartY;
	float	TaxiStartZ;
	float	TaxiMiddleX;
	float	TaxiMiddleY;
	float	TaxiMiddleZ;
	float	TaxiEndX;
	float	TaxiEndY;
	float	TaxiEndZ;
	char	name[MAX_DB_NAME_LEN];
}; // size 0x60

struct dbAirstripInfo
{
	short	sID;            // 0x00
	short	sData02;	    // 0x02 bitfield
	float	fHangarX;       // 0x04
	float	fHangarY;       // 0x08
	float	fHangarZ;       // 0x0C
	float	fRunwayStartX;  // 0x10
	float	fRunwayStartY;  // 0x14
	float	fRunwayStartZ;  // 0x18
	float	fRunwayEndX;    // 0x1C
	float	fRunwayEndY;    // 0x20
	float	fRunwayEndZ;    // 0x24
	float	fTaxiStartX;    // 0x28
	float	fTaxiStartY;    // 0x2C
	float	fTaxiStartZ;    // 0x30
	float	fTaxiMiddleX;   // 0x34
	float	fTaxiMiddleY;   // 0x38
	float	fTaxiMiddleZ;   // 0x3C
	float	fTaxiEndX;      // 0x40
	float	fTaxiEndY;      // 0x44
	float	fTaxiEndZ;      // 0x48
    float   fData4C;        // 0x4C
    float   fData50;        // 0x50
    float   fData54;        // 0x54
    float   fData58;        // 0x58
    float   fData5C;        // 0x5C
    float   fData60;        // 0x60
	char	szName[MAX_DB_NAME_LEN];  // 0x64
    int     nOperation;        // 0x78
}; // size 0x7C
void ConvertSC2DW( dbAirstripInfo* pdbAirstripInfo, scAirstripInfo* pscAirstripInfo );

///////////////////////////////////////////////////////////////////////////////
// ShipBasedAircraft DB
///////////////////////////////////////////////////////////////////////////////
struct scShipBasedAircraftInfo
{
	short	m_sID;//0x00 //unique Index but WHAT FOR ??????
	short	ShipClassPtr;//0x2 	//  Pointer to ShipClass
	bool	m_bHasEntities;//0x4	// always 1
	bool	bData02;// alignment
	
	struct  {
		short EntityPtr;
		short EntityCount;
	}m_EntitySlots[ENTITY_NUM_SHIPBASEDAIRCRAFT];
} ; // size 0x22

struct dbShipBasedAircraftInfo
{
	short	sID;// 0x00 //unique Index but WHAT FOR ??????
	short	sShipClassID;// 0x02 	//  Pointer to ShipClass
	bool	bHasEntities;// 0x04	// always 1
	bool	bData02;// alignment0x05	
	struct
    {
		short sEntityID;    // 0x00
		short sEntityCount;  // 0x02
        short sData04;      // 0x04
        short sData06;      // 0x06
        short sData08;      // 0x08

	}m_EntitySlots[ENTITY_NUM_SHIPBASEDAIRCRAFT];
	
}; // size 0x4C
void ConvertSC2DW( dbShipBasedAircraftInfo* pdbShipBasedAircraftInfo, scShipBasedAircraftInfo* pscShipBasedAircraftInfo );


///////////////////////////////////////////////////////////////////////////////
// Thrust DB
///////////////////////////////////////////////////////////////////////////////
struct dbThrustInfo
{
    unsigned short sID;//0x00
    short sMinRPM;//0x02 former sData02
    short sMaxRPM;// 0x04 former sRPMs
    short sMinPropPitch;// 0x06 former sData06
    short sMaxPropPitch;// 0x08 former sData08
    unsigned char  cType;//0x0A cData0A, ThrustType
    unsigned char  cBaseNoise;//0x0B  former cData0B
    unsigned char  cSpeedNoise;//0x0C former cData0C
    unsigned char  cPropEfficiency;// 0x0D former cData0D
    unsigned char  bAutoCalcEfficiency;// 0x0E former bHasThrottle, former cData0E
    unsigned char  cData0F;//0x0F unused, alignment    
    float fPropDiameter;// 0x10 former fData10
    float fFuelType;//0x14 FuelType 0 == Diesel, 1 == Battery, 2 Nuclear
    float fPosX;//0x18
    float fPosY;//0x1C
    float fPosZ;//0x20
    float fRotXY;//0x24
    float fRotXZ;//0x28
    char  szName[MAX_DB_NAME_LEN];//0x2C
};// 0x40

//QuickMission databases


//struct qmplatforms.eqd
struct dbQMPlatforms
{
};// 0x18

//struct qmtasks.eqd
struct dbQMTasks
{

};// 0x30


void GetSensorAttributes( unsigned _uSensorAttribs, unsigned& _uSensorAttribs1, unsigned& _uSensorAttribs2 );

#pragma pack( pop, before_db_include )
#endif